@@ -1,4 +1,5 @@
 use crate::*;
+use anchor_lang::solana_program::sysvar::instructions::get_instruction_relative;
 use anchor_spl::token;
 use num_traits::ToPrimitive;
 
@@ -62,15 +63,11 @@ impl<'info> Lock<'info> {
             )?;
         }
 
-        // update the escrow
-        let escrow = &mut self.escrow;
-        escrow.amount = unwrap_int!(escrow.amount.checked_add(amount));
-        escrow.escrow_started_at = next_escrow_started_at;
-        escrow.escrow_ends_at = next_escrow_ends_at;
+        // update the escrow and locker
 
-        // update the locker
         let locker = &mut self.locker;
-        locker.locked_supply = unwrap_int!(locker.locked_supply.checked_add(amount));
+        let escrow = &mut self.escrow;
+        escrow.record_lock_event(locker, amount, next_escrow_started_at, next_escrow_ends_at)?;
 
         emit!(LockEvent {
             locker: locker.key(),
@@ -83,6 +80,31 @@ impl<'info> Lock<'info> {
 
         Ok(())
     }
+
+    pub fn check_whitelisted(&self, ra: &[AccountInfo]) -> ProgramResult {
+        invariant!(
+            ra.len() == 2,
+            "program whitelist enabled; please provide whitelist entry and instructions sysvar"
+        );
+        let accounts_iter = &mut ra.iter();
+        let ix_sysvar_account_info = next_account_info(accounts_iter)?;
+        let program_id = get_instruction_relative(0, ix_sysvar_account_info)?.program_id;
+        if program_id == crate::ID {
+            return Ok(());
+        }
+
+        let whitelist_entry_account_info = next_account_info(accounts_iter)?;
+        require!(
+            !whitelist_entry_account_info.data_is_empty(),
+            ProgramNotWhitelisted
+        );
+        let whitelist_entry =
+            Account::<LockerWhitelistEntry>::try_from(whitelist_entry_account_info)?;
+        assert_keys_eq!(whitelist_entry.locker, self.locker.key());
+        assert_keys_eq!(whitelist_entry.program_id, program_id);
+
+        Ok(())
+    }
 }
 
 impl<'info> Validate<'info> for Lock<'info> {
@@ -91,6 +113,7 @@ impl<'info> Validate<'info> for Lock<'info> {
         assert_keys_eq!(self.escrow.tokens, self.escrow_tokens);
         assert_keys_eq!(self.escrow.owner, self.escrow_owner);
         assert_keys_eq!(self.escrow_owner, self.source_tokens.owner);
+
         Ok(())
     }
 }
